@@ -14,7 +14,7 @@ sub e64 {
     my ($arg) = @_;
     my $res = encode_base64($arg);
     chomp($res);
-    return ($res);
+    return $res;
 }
 
 sub SASL {
@@ -110,7 +110,7 @@ sub get_auth_details_plain {
         return;
     }
 
-    return ($loginas, $user, $passClear);
+    return $loginas, $user, $passClear;
 }
 
 sub get_auth_details_login {
@@ -127,7 +127,7 @@ sub get_auth_details_login {
 
     my $passClear = get_base64_response($session, 'Password:') or return;
 
-    return ($user, $passClear);
+    return $user, $passClear;
 }
 
 sub get_auth_details_cram_md5 {
@@ -157,7 +157,7 @@ sub get_auth_details_cram_md5 {
     }
 
     $session->{auth}{ticket} = $ticket;
-    return ($ticket, $user, $passHash);
+    return $ticket, $user, $passHash;
 }
 
 sub get_base64_response {
@@ -188,40 +188,40 @@ sub validate_password {
 
     if (!$src_crypt && !$src_clear) {
         $self->log(LOGINFO, "fail: missing password");
-        return ($deny, "$file - no such user");
+        return $deny, "$file - no such user";
     }
 
     if (!$src_clear && $method =~ /CRAM-MD5/i) {
         $self->log(LOGINFO, "skip: cram-md5 not supported w/o clear pass");
-        return (DECLINED, $file);
+        return DECLINED, $file;
     }
 
     if (defined $attempt_clear) {
         if ($src_clear && $src_clear eq $attempt_clear) {
             $self->log(LOGINFO, "pass: clear match");
-            return (OK, $file);
+            return OK, $file;
         }
 
         if ($src_crypt && $src_crypt eq crypt($attempt_clear, $src_crypt)) {
             $self->log(LOGINFO, "pass: crypt match");
-            return (OK, $file);
+            return OK, $file;
         }
     }
 
     if (defined $attempt_hash && $src_clear) {
         if (!$ticket) {
             $self->log(LOGERROR, "skip: missing ticket");
-            return (DECLINED, $file);
+            return DECLINED, $file;
         }
 
         if ($attempt_hash eq hmac_md5_hex($ticket, $src_clear)) {
             $self->log(LOGINFO, "pass: hash match");
-            return (OK, $file);
+            return OK, $file;
         }
     }
 
     $self->log(LOGINFO, "fail: wrong password");
-    return ($deny, "$file - wrong password");
+    return $deny, "$file - wrong password";
 }
 
 # tag: qpsmtpd plugin that sets RELAYCLIENT when the user authenticates
